@@ -1,6 +1,6 @@
 #include "lsx_processor.h"
 
-#include <lttoolbox/compression.h>
+#include <lttoolbox/file_utils.h>
 #include <cstring>
 
 LSXProcessor::LSXProcessor()
@@ -16,161 +16,56 @@ LSXProcessor::LSXProcessor()
   escaped_chars.insert('@');
   escaped_chars.insert('<');
   escaped_chars.insert('>');
-
-  null_flush = false;
-  dictionary_case = false;
-  at_end = false;
-  at_null = false;
 }
 
 void
 LSXProcessor::load(FILE *input)
 {
-  fpos_t pos;
-  if (fgetpos(input, &pos) == 0) {
-    char header[4]{};
-    if (fread(header, 1, 4, input) == 4 &&
-        strncmp(header, HEADER_LTTOOLBOX, 4) == 0) {
-      auto features = read_le<uint64_t>(input);
-      if (features >= LTF_UNKNOWN) {
-        throw std::runtime_error("FST has features that are unknown to this version of lttoolbox - upgrade!");
-      }
-    }
-    else {
-      // Old binary format
-      fsetpos(input, &pos);
-      // of course, lsx-comp would never generate this...
-    }
-  }
-
-  // letters
-  int len = Compression::multibyte_read(input);
-  while(len > 0)
-  {
-    alphabetic_chars.insert(static_cast<UChar32>(Compression::multibyte_read(input)));
-    len--;
-  }
+  readTransducerSet(input, alphabetic_chars, alphabet, trans);
 
   // symbols
-  alphabet.read(input);
   word_boundary = alphabet("<$>"_u);
   word_boundary_s = alphabet("<$_>"_u);
   word_boundary_ns = alphabet("<$->"_u);
   any_char = alphabet("<ANY_CHAR>"_u);
   any_tag = alphabet("<ANY_TAG>"_u);
 
-  len = Compression::multibyte_read(input);
-  Compression::string_read(input); // name
-  // there should only be 1 transducer in the file
-  // so ignore any subsequent ones
-  trans.read(input, alphabet);
-
-  initial_state.init(trans.getInitial());
-  all_finals = trans.getFinals();
+  for (auto& it : trans) {
+    root.addTransition(0, 0, it.second.getInitial(), 0.0);
+    all_finals.insert(it.second.getFinals().begin(),
+                      it.second.getFinals().end());
+  }
+  initial_state.init(&root);
 }
 
 void
 LSXProcessor::readNextLU(InputFile& input)
 {
-  vector<UString> parts = vector<UString>(3);
-  int loc = 0; // 0 = blank, 1 = bound blank, 2 = LU
-  bool box = false; // are we in a [ ] blank
-  while(!input.eof())
-  {
-    UChar32 c = input.get();
-    if ((unsigned int)c == U_EOF) {
-        break;
-    }
-    if(null_flush && c == '\0')
-    {
-      at_end = true;
-      at_null = true;
-      break;
-    }
-    else if(c == '\\')
-    {
-      parts[loc] += c;
-      c = input.get();
-      parts[loc] += c;
-    }
-    else if(loc == 0 && box)
-    {
-      if(c == ']')
-      {
-        box = false;
-      }
-      parts[loc] += c;
-    }
-    else if(loc == 0 && c == '[')
-    {
-      c = input.get();
-      if(c == '[')
-      {
-        loc = 1;
-      }
-      else
-      {
-        parts[loc] += '[';
-        parts[loc] += c;
-        if(c != ']')
-        {
-          box = true;
-        }
-        if(c == '\\')
-        {
-          parts[loc] += input.get();
-        }
-      }
-    }
-    else if(loc == 1 && c == ']')
-    {
-      c = input.get();
-      if(c == ']')
-      {
-        c = input.get();
-        if(c == '^')
-        {
-          loc = 2;
-        }
-        else
-        {
-          // this situation is invalid
-          // but I like making parsers harder to break than required
-          // by the standard
-          parts[loc] += "]]"_u;
-          parts[loc] += c;
-        }
-      }
-      else
-      {
-        parts[loc] += ']';
-        parts[loc] += c;
-        if(c == '\\')
-        {
-          parts[loc] += input.get();
-        }
-      }
-    }
-    else if(loc == 0 && c == '^')
-    {
-      loc = 2;
-    }
-    else if(loc == 2 && c == '$')
-    {
-      break;
-    }
-    else
-    {
-      parts[loc] += c;
-    }
+  UString blank, wblank, lu;
+  blank = input.readBlank();
+  if (input.peek() == '[') {
+    input.get();
+    input.get();
+    wblank = input.finishWBlank();
+    wblank = wblank.substr(2, wblank.size()-4);
   }
-  if(input.eof())
-  {
+  if (input.peek() == '^') {
+    input.get();
+    lu = input.readBlock('^', '$');
+    lu = lu.substr(1, lu.size()-2);
+  }
+
+  blank_queue.push_back(blank);
+  bound_blank_queue.push_back(wblank);
+  lu_queue.push_back(lu);
+
+  if (input.peek() == '\0') {
+    input.get();
+    at_end = true;
+    at_null = true;
+  } else if (input.eof()) {
     at_end = true;
   }
-  blank_queue.push_back(parts[0]);
-  bound_blank_queue.push_back(parts[1]);
-  lu_queue.push_back(parts[2]);
 }
 
 void
@@ -191,8 +86,7 @@ LSXProcessor::processWord(InputFile& input, UFILE* output)
   }
   size_t last_final = 0;
   UString last_final_out;
-  State s;
-  s.init(trans.getInitial());
+  State s = initial_state;
   size_t idx = 0;
   bool firstupper = false;
   bool uppercase = false;
@@ -212,8 +106,8 @@ LSXProcessor::processWord(InputFile& input, UFILE* output)
       break;
     }
     if (idx == 0 && !dictionary_case) {
-      firstupper = iswupper(lu[0]);
-      uppercase = lu.size() > 1 && iswupper(lu[1]);
+      firstupper = u_isupper(lu[0]);
+      uppercase = lu.size() > 1 && u_isupper(lu[1]);
     }
     for(size_t i = 0; i < lu.size(); i++)
     {
@@ -286,7 +180,7 @@ LSXProcessor::processWord(InputFile& input, UFILE* output)
       {
         wblank += "; "_u;
       }
-      
+
       wblank += bound_blank_queue[i];
     }
   }
@@ -299,33 +193,34 @@ LSXProcessor::processWord(InputFile& input, UFILE* output)
   size_t pos = 0;
   bool pop_queue = true;
   bool replace_empty = false;
+  std::vector<UString> blanks;
+  std::vector<UString> lus;
   while(pos != UString::npos && pos != last_final_out.size())
   {
     if (pop_queue) {
       if (output_count < last_final) {
-        write(blank_queue[output_count], output);
         if (replace_empty && blank_queue[output_count].empty()) {
-          u_fputc(' ', output);
+          blanks.push_back(" "_u);
+        } else {
+          blanks.push_back(blank_queue[output_count]);
         }
         output_count++;
       } else {
-        u_fputc(' ', output);
+        blanks.push_back(" "_u);
       }
+    } else {
+      blanks.push_back(""_u);
     }
-    write(wblank, output);
-    u_fputc('^', output);
     size_t start = pos;
     pos = last_final_out.find("<$"_u, start);
     if(pos == UString::npos)
     {
-      write(last_final_out.substr(start), output);
-      u_fputc('$', output);
+      lus.push_back(last_final_out.substr(start));
       break;
     }
     else
     {
-      write(last_final_out.substr(start, pos-start), output);
-      u_fputc('$', output);
+      lus.push_back(last_final_out.substr(start, pos-start));
       pos += 2;
       if (last_final_out[pos] == '-') {
         pop_queue = false;
@@ -341,17 +236,33 @@ LSXProcessor::processWord(InputFile& input, UFILE* output)
       pos++;
     }
   }
+  UString trail_blank;
   for(; output_count < last_final; output_count++)
   {
     if(blank_queue[output_count] != " "_u)
     {
-      write(blank_queue[output_count], output);
+      trail_blank += blank_queue[output_count];
     }
   }
 
   blank_queue.erase(blank_queue.begin(), blank_queue.begin()+last_final);
   bound_blank_queue.erase(bound_blank_queue.begin(), bound_blank_queue.begin()+last_final);
   lu_queue.erase(lu_queue.begin(), lu_queue.begin()+last_final);
+
+  if (repeat_rules) {
+    blank_queue.front() = trail_blank + blank_queue.front();
+    blank_queue.insert(blank_queue.begin(), blanks.begin(), blanks.end());
+    if (!wblank.empty()) {
+      wblank = wblank.substr(2, wblank.size()-4);
+    }
+    bound_blank_queue.insert(bound_blank_queue.begin(), lus.size(), wblank);
+    lu_queue.insert(lu_queue.begin(), lus.begin(), lus.end());
+  } else {
+    for (size_t i = 0; i < lus.size(); i++) {
+      u_fprintf(output, "%S%S^%S$", blanks[i].c_str(), wblank.c_str(), lus[i].c_str());
+    }
+    write(trail_blank, output);
+  }
 }
 
 void
