@@ -12,20 +12,22 @@
 class LSXProcessor
 {
 private:
-  TransExe trans;
+  Node root;
+  std::map<UString, TransExe> trans;
   State initial_state;
-  set<UChar32> escaped_chars;
-  set<UChar32> alphabetic_chars;
-  map<Node *, double> all_finals;
+  std::set<UChar32> escaped_chars;
+  std::set<UChar32> alphabetic_chars;
+  std::map<Node *, double> all_finals;
   Alphabet alphabet;
-  bool null_flush;
-  bool dictionary_case;
-  bool at_end;
-  bool at_null;
+  bool null_flush = true;
+  bool dictionary_case = false;
+  bool repeat_rules = false;
+  bool at_end = false;
+  bool at_null = false;
 
-  deque<UString> blank_queue;
-  deque<UString> bound_blank_queue;
-  deque<UString> lu_queue;
+  std::deque<UString> blank_queue;
+  std::deque<UString> bound_blank_queue;
+  std::deque<UString> lu_queue;
 
   void readNextLU(InputFile& input);
   void processWord(InputFile& input, UFILE* output);
@@ -47,6 +49,10 @@ public:
   {
     dictionary_case = val;
   }
+  void setRepeatMode(bool val)
+  {
+    repeat_rules = val;
+  }
 };
 
 #endif
