@@ -102,3 +102,54 @@ class SpaceManipulation2(unittest.TestCase, ProcTest):
                        "^a<ex>$^,<cm>$ ^b<ir>$",
                        "^a<ex>$^,<cm>$ _ ^b<ir>$",
                        "^a<ex>$^,<cm>$_^b<ir>$"]
+
+
+class Empty(unittest.TestCase, ProcTest):
+    procdir = "lr"
+    procdix = "data/empty.lsx"
+    inputs = ["^c<ex>$ ^d<ir>$"]
+    expectedOutputs = ["^c<ex>$ ^d<ir>$"]
+
+class Variant1(unittest.TestCase, ProcTest):
+    procdix = "data/variants.dix"
+    compflags = ['-v', 'abc']
+    inputs = ['^take<vblex><pres>$ ^up<adv>$',
+              '^take<vblex><pres>$ ^out<adv>$',
+              '^take<vblex><pres>$ ^over<adv>$']
+    expectedOutputs = ['^take# up<vblex><pres>$',
+                       '^take# out<vblex><pres>$',
+                       '^take<vblex><pres>$ ^over<adv>$']
+
+class Variant2(unittest.TestCase, ProcTest):
+    procdix = "data/variants.dix"
+    compflags = ['-v', 'xyz']
+    inputs = ['^take<vblex><pres>$ ^up<adv>$',
+              '^take<vblex><pres>$ ^out<adv>$',
+              '^take<vblex><pres>$ ^over<adv>$']
+    expectedOutputs = ['^take# up<vblex><pres>$',
+                       '^take<vblex><pres>$ ^out<adv>$',
+                       '^take# over<vblex><pres>$']
+
+
+class Weights(unittest.TestCase, ProcTest):
+    procdix = "data/weights.lsx"
+    inputs = ['^turn<vblex><pres>$ ^off<adv>$',
+              '^turn<vblex><pres>$ ^things<n>$ ^off<adv>$',
+              '^turn<vblex><pres>$ ^the<det>$ ^thing<n>$ ^off<adv>$',
+              '^turn<vblex><pres>$ ^me<prn><pers>$ ^off<adv>$',
+              ]
+    expectedOutputs = ['^deactivate<vblex><pres>$',
+                       '^deactivate<vblex><pres>$ ^things<n>$',
+                       '^deactivate<vblex><pres>$ ^the<det>$ ^thing<n>$',
+                       '^alienate<vblex><pres>$ ^me<prn><pers>$',
+                       ]
+
+class Repeat(unittest.TestCase, ProcTest):
+    procdix = "data/repeat.lsx"
+    procflags = ['-z', '-r']
+    inputs = ['[1] ^hot<adj>$ ^dog<n><sg>$',
+              '[2] ^take<vblex><pres>$ ^the<det><def><sp>$ ^silver<adj>$ ^dollar<n><sg>$ ^out<adv>$',
+              '[3] ^take<vblex><pres>$ ^the<det><def><sp>$ ^hot<adj>$ ^dog<n><sg>$ ^out<adv>$']
+    expectedOutputs = ['[1] ^hot# dog<n><sg>$',
+                       '[2] ^take# out<vblex><pres>$ ^the<det><def><sp>$ ^silver<adj>$ ^dollar<n><sg>$',
+                       '[3] ^take# out<vblex><pres>$ ^the<det><def><sp>$ ^hot# dog<n><sg>$']
